/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemExitHook.java 1:04:18 PM ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.system.launch;

import static org.objectweb.petals.topology.generated.DomainMode.STANDALONE;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Properties;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBException;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.petals.topology.TopologyBuilder;
import org.objectweb.petals.topology.TopologyException;
import org.objectweb.petals.topology.generated.Container;
import org.objectweb.petals.topology.generated.Topology;
import org.ow2.petals.PetalsException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.server.PetalsListener;
import org.ow2.petals.kernel.server.PetalsServer;
import org.ow2.petals.tools.jmx.api.PetalsJMXClient;
import org.ow2.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminServiceErrorException;

/**
 * This class is used to start and manage the Petals platform from command line
 * 
 * @author alouis, ddesjardins, Rafael Marins, rnaudin
 * @version $Rev: 461 $ $Date: 2006-05-24 14:12:45 +0200 (mer., 24 mai 2006) $
 */
public class PetalsStandaloneLauncher implements PetalsListener {

    public static final String START_COMMAND = "start";

    public static final String STOP_COMMAND = "stop";

    public static final String SHUTDOWN_COMMAND = "shutdown";

    public static final String VERSION_COMMAND = "version";

    public static final String JNDIBROWSE_COMMAND = "JNDIBrowser";

    public PetalsStandaloneLauncher() {
        super();
    }

    /**
     * The launcher instance to process the main program
     */
    private static PetalsStandaloneLauncher launcher;

    /**
     * The Petals server instance
     */
    private static PetalsServer petalsServer;

    /**
     * System exit hook used if the command line is used
     */
    private static SystemExitHook systemExitHook;

    /**
     * @param args
     */
    public static void main(String[] args) {

        // Process the Command and exits abnormaly if necessary

        String command = null;
        try {
            launcher = new PetalsStandaloneLauncher();

            petalsServer = new PetalsServer();

            command = (args.length == 0 || args[0].startsWith("-")) ? START_COMMAND : args[0];

            if (STOP_COMMAND.equals(command)) {
                System.out.println("Petals is stopping...");
                PetalsJMXClient petalsJmxClient = launcher.getPetalsJmxClient();
                petalsJmxClient.getPetalsAdminServiceClient().stopContainer();
            } else if (SHUTDOWN_COMMAND.equals(command)) {
                System.out.println("Petals is stopping...");
                PetalsJMXClient petalsJmxClient = launcher.getPetalsJmxClient();
                petalsJmxClient.getPetalsAdminServiceClient().shutdownContainer();
            } else if (VERSION_COMMAND.equals(command)) {
                PetalsJMXClient petalsJmxClient = launcher.getPetalsJmxClient();
                System.out.println(petalsJmxClient.getAdminServiceClient().getSystemInfo());
            } else if (JNDIBROWSE_COMMAND.equals(command)) {
                System.out.println(petalsServer.browseJNDI());
            } else if (START_COMMAND.equals(command)) {

                // show the fractal explorer mode if asked
                if (args.length > 0 && "-explore".equalsIgnoreCase(args[0])) {
                    petalsServer.setObserver(true);
                }

                // check if started or not stopped correctly
                if (petalsServer.isLocked()) {
                    throw new PetalsException(
                            "Can not start the PEtALS server, remove lock file or stop server");
                }

                petalsServer.addStopListener(launcher);

                // add a hook if a terminate signal is sent from the command
                // line
                systemExitHook = new SystemExitHook(petalsServer);
                Runtime.getRuntime().addShutdownHook(systemExitHook);

                System.out.println("Petals is starting...");
                petalsServer.init(false);
                petalsServer.start();

                // show the commandLine mode if asked
                if (args.length > 0 && "-console".equalsIgnoreCase(args[0])) {
                    launcher.commandLineMode();
                }
            }
        } catch (Throwable e) {
            System.err.println("Command processing error: " + command);
            e.printStackTrace();
            System.exit(-1);
        }
    }

    private PetalsJMXClient getPetalsJmxClient() throws IOException, JAXBException,
            TopologyException, ConnectionErrorException, PetalsAdminDoesNotExistException,
            PetalsAdminServiceErrorException, ConfigurationException {

        // retrieve resources
        URL serverPropsURL = this.getClass().getResource(ConfigurationService.SERVER_PROPS_FILE);
        if (serverPropsURL == null) {

            throw new IOException("Failed to reach the resource '"
                    + ConfigurationService.SERVER_PROPS_FILE + "'");
        }
        File serverPropsFile = new File(serverPropsURL.getPath());
        URL topologyURL = this.getClass().getResource(ConfigurationService.TOPOLOGY_FILE);
        if (topologyURL == null) {
            throw new IOException("Failed to reach the resource '"
                    + ConfigurationService.TOPOLOGY_FILE + "'");
        }
        File topologyFile = new File(topologyURL.getPath());

        // build the topology object
        TopologyBuilder topologyBuilder = TopologyBuilder.getInstance();
        Topology topology = topologyBuilder.createTopology(topologyFile);

        // retrieve the server properties
        Properties serverProperties = new Properties();
        serverProperties.load(new FileInputStream(serverPropsFile));

        String containerName = serverProperties
                .getProperty(ConfigurationService.PROPERTY_CONTAINER_NAME);
        if (containerName == null) {
            containerName = System.getProperty(ConfigurationService.PROPERTY_CONTAINER_NAME);
        }

        Container localContainer = null;
        if (STANDALONE.equals(topology.getDomain().getMode())) {
            localContainer = topology.getContainers().getContainer().get(0);
        } else {
            for (Container container : topology.getContainers().getContainer()) {
                if (container.getName().equals(containerName)) {
                    localContainer = container;
                    break;
                }
            }
        }

        if (localContainer == null) {
            throw new ConfigurationException("Cannot retrieve the local container configuration");
        }

        PetalsJMXClient petalsJMXClient = new PetalsJMXClient(localContainer.getHost(),
                localContainer.getConfiguration().getJmxservice().getRmiport(), localContainer
                        .getConfiguration().getJmxservice().getUser(), localContainer
                        .getConfiguration().getJmxservice().getPassword());

        return petalsJMXClient;

    }

    /**
     * Show a command line to interact with Petals. TODO we pass the PetalsAdmin
     * object to the CommandReader to perform some actions. Find a better way.
     * 
     * @throws TopologyException
     * @throws JAXBException
     * @throws IOException
     * @throws PetalsAdminServiceErrorException
     * @throws PetalsAdminDoesNotExistException
     * @throws ConnectionErrorException
     * @throws ConfigurationException
     * @throws NoSuchInterfaceException
     * @throws ADLException
     * @throws PetalsException
     */
    protected void commandLineMode() throws ConnectionErrorException,
            PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException, IOException,
            JAXBException, TopologyException, ConfigurationException {
        PetalsJMXClient petalsJmxClient = launcher.getPetalsJmxClient();
        CommandReader console = new CommandReader(petalsJmxClient.getPetalsAdminServiceClient(),
                petalsServer);
        console.read();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.kernel.server.PetalsListener#onPetalsStarted()
     */
    public void onPetalsStarted() {
        System.out.println("##### Petals container correctly started #####");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.kernel.server.PetalsListener#onPetalsStopped()
     */
    public void onPetalsStopped() {
        // remove the System Hook to avoid the Petals server stop() called a
        // second time
        Runtime.getRuntime().removeShutdownHook(systemExitHook);
        System.out.println("Petals is stopped");

        System.exit(0);
    }
}
